--[[
Copyright (C) 2016-2017  Neemys

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

See <LICENSE.txt> for a copy of this license, or see <http://www.gnu.org/licenses/>
]]






require("mods-config.load-all") --load for all requested platform the default config concerning blocked building, include mod compatibility modification




local tiles = data.raw["tile"];

--will do the necessary change to speed for this generation according to config
if(settings.startup[buildingplatform.constants.platformGeneratorSetting].value == "concentric-circle") then
	--player speed penalty
	if(settings.startup[buildingplatform.constants.speedPenaltySetting].value) then
		data.raw.character.character.running_speed = data.raw.character.character.running_speed * .65 --35% speed reduction
	end

	--non ground tile speed buff
	if(settings.startup[buildingplatform.constants.tilesSpeedBonusSetting].value) then
		local excludedTile
		if(settings.startup[buildingplatform.constants.speedPlatformSpeedBonusSetting].value) then  --if speed platform has its own bonus, do not apply the overall bonus as well
			excludedTile = tiles[buildingplatform.constants.speedTileName]
		else
			excludedTile = nil
		end
		local groundSpeed = nil
		if tiles['grass'] then
			groundSpeed = tiles['grass'].walking_speed_modifier
		end
		if not groundSpeed then
			groundSpeed = 1
		end
		for _,tile in pairs(tiles) do
			if tile ~= excludedTile and tile.walking_speed_modifier and tile.walking_speed_modifier > groundSpeed then
				tile.walking_speed_modifier = tile.walking_speed_modifier * 1.45 --45% speed bonus
			end
		end
	end
end



---- Block all selected building added by all mod during data-updates phase.

-- By adding non craftable tile and restricted building the same collision_mask, the game make building footprint red outside building platform
-- Can't do the same with placeable tile as when tile are  placed and collide with something they just remove it without compensation. So a script take take of placeable tile check (see controls.lua). We don't add the collision to platform to avoid blocking a building completly (if not placeable outside A and outside B, then it's not placeable anywhere)

--just add the given layer to the item's collision mask
local function addLayerToCollision(item,layer)
	if not item  then return end
	if item.collision_mask then
		table.insert(item.collision_mask, layer);
	else
		item.collision_mask = {layer,"water-tile","player-layer"} --Most building does not have collision_mask by default and use the game engine to simulate one, so we need to manually add collision with player and water. Moreover when setting a mask we need to have at least two collision layer in it or they don't use it (for performance reason said a dev, reported as not a bug (link to thread?)).
	end
end

-- add a dummy entity with only purpose is to give control.lua information.
local function addDummyEntity(dummyName,infoToConvey)
	data:extend({{
		type = "flying-text",
		name = dummyName,
		time_to_live = 0,
		speed = 0,
		order = infoToConvey
	}})
end

--get all platform name and aliases to be able to make them not block each other buildings.
local platformNames = {}
for _,platformDefinition in pairs(buildingplatform.tiles) do
	platformNames[platformDefinition.name] = true
	if platformDefinition.aliases then
		for _,alias in pairs(platformDefinition.aliases) do
			platformNames[alias] = true
		end
	end
end

for _,platformDefinition in pairs(buildingplatform.tiles) do
	--before doing anything we must add if necessary the exceptions that should override any other rule added by any mod. Those exception are for mod compatibility.
	-- TODO : move those exceptions elsewhere
	-- exception for Reactor mod, if one of those entity is edited, then the game draw them in wrong order, hidding the interface making it non interactable. They are non placeable entity so it don't change anything gameplay wise.
	if buildingplatform.detectedMods.reactors then
		if(platformDefinition.buildingsAllowed["storage-tank"]) then
			buildingplatform.blockAllBut("storage-tank","reactor%-boiler",platformDefinition)
		end
		if(platformDefinition.buildingsAllowed["constant-combinator"]) then
			buildingplatform.blockAllBut("constant-combinator","reactor%-interface",platformDefinition)
		end
	end
	--Exception for AAI programmable vehicles mod. If one of those entity is edited, then the game draw them in wrong order, hidding it and making it non interactable. They are non placeable entity so it don't change anything gameplay wise.
	if buildingplatform.detectedMods.aaiProgrammableVehicles then
		if(platformDefinition.buildingsAllowed["radar"]) then
			buildingplatform.blockAllBut("radar","vehicle%-deployer%-belt",platformDefinition)
		end
	end

	--add the dummy entity that will tell control.lua which platform have which layer. When and if dev team implement a better way to pass information to control.lua, that part should be changed
	addDummyEntity(buildingplatform.constants.dummyPrefix..platformDefinition.name,platformDefinition.layer)
	if platformDefinition.aliases then
		for _,alias in pairs(platformDefinition.aliases) do
			addDummyEntity(buildingplatform.constants.dummyPrefix..alias,platformDefinition.layer)
		end
	end

	--Add collision to mask for non mineable tile (except platforms). All mineable (and placeable) tile are handled by a function in control.lua
	for _,item in pairs(tiles) do
		if not item.minable and not platformNames[item.name] then
			addLayerToCollision(item,platformDefinition.layer);
		end
	end


	--add to all building marked as blocked the mask to block their placement.

	--"block all but" rules
	for name,tablePattern in pairs(platformDefinition.buildingsAllowed) do
		local list = data.raw[name];
		if #tablePattern == 0 then --no pattern at all, block all
			for name,item in pairs(list) do
				addLayerToCollision(item,platformDefinition.layer);
			end
		else
			for itemName,item in pairs(list) do
				local matchAtLeastOneTime = false
				for _,pattern in pairs(tablePattern) do
					if( string.match(itemName,pattern) == itemName) then
						matchAtLeastOneTime = true
					end
				end
				if(not matchAtLeastOneTime) then
					addLayerToCollision(item,platformDefinition.layer);
				end
			end
		end
	end

	--"block only" rules
	for name,tablePattern in pairs(platformDefinition.buildingsDisallowed) do
		if(not platformDefinition.buildingsAllowed[name]) then --if all, or most building are blocked, "block only" rules are ignored.
			local list = data.raw[name];
			for itemName,item in pairs(list) do
				local matchAtLeastOneTime = false
				for _,pattern in pairs(tablePattern) do
					if( string.match(itemName,pattern) == itemName) then
						matchAtLeastOneTime = true
					end
					if(matchAtLeastOneTime) then
						addLayerToCollision(item,platformDefinition.layer);
					end
				end
			end
		end
	end
end

if (settings.startup[buildingplatform.constants.blockTileOnPlatformSetting].value) then
	--Use layer-11 collision layer to block tile placement on platforms
	for _,tile in pairs(tiles) do
		if platformNames[tile.name] then
			addLayerToCollision(tile,"layer-11");
		end
		if(tile.minable) then
			local item = data.raw.item[tile.minable.result]
			if item and item.place_as_tile then
				table.insert(item.place_as_tile.condition,"layer-11")
			end
		end
	end
end
