--[[
Copyright (C) 2016-2017  Neemys

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

See <LICENSE.txt> for a copy of this license, or see <http://www.gnu.org/licenses/>
]]



if not buildingplatform.tiles then buildingplatform.tiles = {} end

local howManyCall = 0 --lua don't have static, so at least keep track using a local to file variable
local layerList = {"layer-15","layer-14","layer-13","layer-12"} --take the far most layer first in case the game use up some layer, it will not mess with most use of our function and the mod will continue to work until it is updated.


--[[
TODO : make platform that have no rules to don't take a layer.
TODO : parameter verification
]]

--[[API_DOC
@description Add a platform that can block building to the game.

Can only be called 4 times. It will throw an error on the fifth call. This number counts the default platform call as well.
@param tileName string Name that should be given to the tile, used in tile definition and noise layer name (if needed).
@param initAutoplace boolean If true, then the autoplace object will be added to the tile, autoplace-control and noise-layer will be added to data. If you don't want to use the game's default resource placement for placing this platform then you need to set this to false.
@param tileAutoplaceOrder string Factorio formatted order to tell the game where to put the autoplace config line in new world screen (only if initAutoplace is true).
@param tileAutoplaceObject table Factorio autoplace object to add to the tile (only if initAutoplace is true).
@param tileMapColor table Factorio color object, map_color object a tile definition need.
@param tileFunctionToEditTileDefinition function one parameter : the factorio tile definition created for the platform. If you need to do specific modification to the tile definition you can do them in this function. This function is called after the tile definition is populated, but before it is added to factorio data. You can't edit the tile name here.
@param tileAliases (table of strings) name of other tile that should behave as this platform and allow the same buildings. Those tiles should already exist.
@param tint tint value

@return a table (tileObject) that can be given to other API function to add rules to this platform. You shouldn't edit that object by yourself.
In case you need that object somewhere where you can't pass the object returned, you can get it in buildingplatform.tiles[yourTileName].
@rework This function need a complete rework to allow more than 4 call, allow more picture customization and other things. It may have no backward compatibility in the future when it is done.
@after data
@before data-updates
]]--


buildingplatform.addPlatform = function(tileName,initAutoplace,tileAutoplaceOrder,tileAutoplaceObject,tileMapColor,tileFunctionToEditTileDefinition,tileAliases,tint)
	howManyCall = howManyCall + 1
	if howManyCall >= 5 then
		error("From building-platform mod, you can't add more than 4 platforms for the moment.")
	end
	if buildingplatform.tiles[tileName] then
		error("From building-platform mod, trying to add a duplicate platform (same name).")
	end
	tile = {
		name=tileName,
		needToInitAutoplace = initAutoplace,
		autoplaceOrder = tileAutoplaceOrder,
		autoplaceObject = tileAutoplaceObject,
		mapColor = tileMapColor,
		tileFunction = tileFunctionToEditTileDefinition,
		buildingsDisallowed = {},
		buildingsAllowed = {},
		layer=layerList[howManyCall],
		aliases=tileAliases,
		tint=tint
	}
	buildingplatform.tiles[tileName] = tile
	return tile
end