--[[
Copyright (C) 2016-2017  Neemys

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

See <LICENSE.txt> for a copy of this license, or see <http://www.gnu.org/licenses/>
]]

--[[
Modify vanilla radar to scan further at a higher energy cost and scan each chunk at a faster pace.
See actual number in comment below.

Should scan a new sector every 16.66666 seconds (vanilla is 33.3 seconds)
]]

--only add radar if building platform tweaks not present and the user activated it in config.
if(settings.startup[buildingplatform.constants.addLongRangeRadarSetting].value and not buildingplatformtweaks) then

	local buildingPlatformRadarRecipe
	if(data.raw["recipe"]["radar"]) then --copy actual concrete data if accessible
		buildingPlatformRadarRecipe = table.deepcopy(data.raw["recipe"]["radar"])
	else --last radar recipe info check in version 0.13.20
		buildingPlatformRadarRecipe = {
			type = "recipe",
			ingredients =
			{
				{"electronic-circuit", 5},
				{"iron-gear-wheel", 5},
				{"iron-plate", 10}
			},
			requester_paste_multiplier = 4
		}
	end

	buildingPlatformRadarRecipe.name = buildingplatform.constants.radarRecipeName
	--since 0.15, recipe can have variant or not, if a mod edit radar recipe, we need to edit value in variant or not
	if(buildingPlatformRadarRecipe.normal) then --variants
		buildingPlatformRadarRecipe.normal.result = buildingplatform.constants.radarName
		buildingPlatformRadarRecipe.normal.enabled = true
		buildingPlatformRadarRecipe.normal.hidden = false
		buildingPlatformRadarRecipe.normal.results = nil
		buildingPlatformRadarRecipe.normal.result_count = 1
		if(buildingPlatformRadarRecipe.expensive) then --also edit expensive variant if present
			buildingPlatformRadarRecipe.expensive.result = buildingplatform.constants.radarName
			buildingPlatformRadarRecipe.expensive.enabled = true
			buildingPlatformRadarRecipe.expensive.hidden = false
			buildingPlatformRadarRecipe.expensive.results = nil
			buildingPlatformRadarRecipe.expensive.result_count = 1
		end
	else --no variant
		buildingPlatformRadarRecipe.name = buildingplatform.constants.radarRecipeName
		buildingPlatformRadarRecipe.result = buildingplatform.constants.radarName
		buildingPlatformRadarRecipe.enabled = true
		buildingPlatformRadarRecipe.hidden = false
		buildingPlatformRadarRecipe.results = nil
		buildingPlatformRadarRecipe.result_count = 1
	end

	local buildingPlatformRadarItem
	if(data.raw["item"]["radar"]) then --copy actual concrete data if accessible
		buildingPlatformRadarItem = table.deepcopy(data.raw["item"]["radar"])
	else --last radar recipe info check in version 0.13.20
		buildingPlatformRadarItem = {
			type = "item",
			subgroup = "defensive-structure",
			order = "d[radar]-b[bp-radar]",
			stack_size = 50
		}
	end
	buildingPlatformRadarItem.name = buildingplatform.constants.radarName
	buildingPlatformRadarItem.place_result = buildingplatform.constants.radarName
	buildingPlatformRadarItem.icon = buildingplatform.constants.modRoot.."/graphics/radar/radar.png"
	buildingPlatformRadarItem.icon_size = 32


	local buildingPlatformRadarEntity
	if(data.raw["radar"]["radar"]) then --copy actual concrete data if accessible
		buildingPlatformRadarEntity = table.deepcopy(data.raw["radar"]["radar"])
	else --last radar recipe info check in version 0.13.20
		buildingPlatformRadarEntity = {
			type = "radar",
			flags = {"placeable-player", "player-creation"},
			minable = {hardness = 0.2, mining_time = 0.5},
			max_health = 150,
			corpse = "big-remnants",
			resistances =
			{
			  {
				type = "fire",
				percent = 70
			  }
			},
			collision_box = {{-1.4, -1.4}, {1.4, 1.4}},
			selection_box = {{-1.5, -1.5}, {1.5, 1.5}},
			energy_source =
			{
			  type = "electric",
			  usage_priority = "secondary-input"
			},
			pictures =
			{
			  priority = "low",
			  width = 153,
			  height = 131,
			  apply_projection = false,
			  direction_count = 64,
			  line_length = 8,
			  shift = {0.875, -0.34375}
			},
			vehicle_impact_sound =  { filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65 },
			working_sound =
			{
			  sound = {
				{
				  filename = "__base__/sound/radar.ogg"
				}
			  },
			  apparent_volume = 2,
			}
		}
	end
	buildingPlatformRadarEntity.name = buildingplatform.constants.radarName
	buildingPlatformRadarEntity.place_result = buildingplatform.constants.radarName
	buildingPlatformRadarEntity.minable.result = buildingplatform.constants.radarName
	buildingPlatformRadarEntity.icon = buildingplatform.constants.modRoot.."/graphics/radar/radar.png"
	buildingPlatformRadarEntity.icon_size = 32
	buildingPlatformRadarEntity.pictures.filename = buildingplatform.constants.modRoot.."/graphics/radar/radar_entity.png"

	buildingPlatformRadarEntity.energy_per_sector = "15MJ" --50% more than vanilla in  0.14
	buildingPlatformRadarEntity.max_distance_of_sector_revealed = "28" -- 100% more than vanilla (0.14)
	buildingPlatformRadarEntity.max_distance_of_nearby_sector_revealed = 0 -- no nearby scan, it's only for long range
	buildingPlatformRadarEntity.energy_per_nearby_scan = "0kJ"-- no energy needed
	buildingPlatformRadarEntity.energy_usage = "1000kW" --233% more than vanilla

	data:extend({buildingPlatformRadarEntity,buildingPlatformRadarItem,buildingPlatformRadarRecipe})
end
