--[[
Copyright (C) 2017  Neemys

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

See <LICENSE.txt> for a copy of this license, or see <http://www.gnu.org/licenses/>
]]

require("constants")

data:extend({
	{
		type = "bool-setting",
		name = buildingplatform.constants.craftableSetting,
		setting_type = "startup",
		default_value = false,
		order = "a-a"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.addLongRangeRadarSetting,
		setting_type = "startup",
		default_value = false,
		order = "b-a"
	},
	{
		type = "string-setting",
		name = buildingplatform.constants.platformGeneratorSetting,
		setting_type = "startup",
		default_value = "default",
		allowed_values = {"default","concentric-circle","grid"},
		order = "e-a"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.speedPenaltySetting,
		setting_type = "startup",
		default_value = true,
		order = "e-b"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.tilesSpeedBonusSetting,
		setting_type = "startup",
		default_value = true,
		order = "e-c"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.speedPlatformSpeedBonusSetting,
		setting_type = "startup",
		default_value = true,
		order = "e-d"
	},
	{
		type = "int-setting",
		name = buildingplatform.constants.spacingBetweenPlatformSetting,
		setting_type = "startup",
		minimum_value = 0,
		default_value = 100,
		order = "e-e"
	},
	{
		type = "int-setting",
		name = buildingplatform.constants.sideLengthSetting,
		setting_type = "startup",
		minimum_value = 1,
		default_value = 40,
		order = "e-f"
	},
	{
		type = "string-setting",
		name = buildingplatform.constants.recipeSetting,
		setting_type = "startup",
		default_value = "normal",
		allowed_values = {"normal","normal-expensive","electronic","electronic-expensive","cheap"},
		order = "a-b"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.unrestrictPowerSetting,
		setting_type = "startup",
		default_value = false,
		order = "c-a"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.unrestrictBoilerSetting,
		setting_type = "startup",
		default_value = false,
		order = "c-b"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.unrestrictCombinatorSetting,
		setting_type = "startup",
		default_value = true,
		order = "c-c"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.unrestrictRoboportSetting,
		setting_type = "startup",
		default_value = true,
		order = "c-d"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.unrestrictLogisticSetting,
		setting_type = "startup",
		default_value = true,
		order = "c-e"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.unrestrictStorageTankSetting,
		setting_type = "startup",
		default_value = false,
		order = "c-f"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.unrestrictRadarSetting,
		setting_type = "startup",
		default_value = false,
		order = "c-h"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.unrestrictSubstationSetting,
		setting_type = "startup",
		default_value = false,
		order = "c-i"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.unrestrictFurnaceSetting,
		setting_type = "startup",
		default_value = false,
		order = "c-j"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.unrestrictAssemblingMachineSetting,
		setting_type = "startup",
		default_value = false,
		order = "c-k"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.unrestrictLabSetting,
		setting_type = "startup",
		default_value = false,
		order = "c-l"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.unrestrictBeaconSetting,
		setting_type = "startup",
		default_value = false,
		order = "c-m"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.unrestrictRocketSiloSetting,
		setting_type = "startup",
		default_value = false,
		order = "c-n"
	},
	{
		type = "bool-setting",
		name = buildingplatform.constants.blockTileOnPlatformSetting,
		setting_type = "startup",
		default_value = false,
		order = "b-c"
	}
})

--mods specific configs
-- Only add enabled mods's config entry
local modsConfigDefaultValues = {
	flowControl = true,
	alertCombinator = true,
	reactors = true,--reactor unblocked because of a bug, see the forum thread for more information
	circuitNetworkSwitch = true,
	factorissimo2 = false,
	highVoltage = true,
	warehousing = false,
	teleportation = false,
	stoneWaterWell = false,
	waterWell = false,
	aaiIndustry = false,
	aaiProgrammableStructures = false,
	aaiProgrammableVehicles = false,
	quarry = false
}
buildingplatform.detectedMods = {}
local iOrder = 1 --to make config value ordered in the same order everytime, make order value not the same
for id,name in pairs(buildingplatform.constants.modIdToName) do
	if(mods[name]) then
		buildingplatform.detectedMods[id]=true
		data:extend({
			{
				type = "bool-setting",
				name = buildingplatform.constants.prefix..id,
				setting_type = "startup",
				default_value = modsConfigDefaultValues[id],
				order = "g-"..iOrder
			}
	})
		iOrder = iOrder + 1
	end
end

--Factorissimo 2 have a special config for itself
if(buildingplatform.detectedMods.factorissimo2) then
	data:extend({
		{
			type = "bool-setting",
			name = buildingplatform.constants.factorissimo2FactoryAsPlatformSetting,
			setting_type = "startup",
			default_value = true,
			order = "g-"..iOrder
		}
	})
	iOrder = iOrder + 1
end

--AAI Programmable Vehicles have a special config for itself
if(buildingplatform.detectedMods.aaiProgrammableVehicles) then
	data:extend({
		{
			type = "bool-setting",
			name = buildingplatform.constants.aaiProgrammableVehiclesUnrestrictDepotSetting,
			setting_type = "startup",
			default_value = true,
			order = "g-"..iOrder
		}
	})
	iOrder = iOrder + 1
end
